"""
User mutations
"""

from typing import Optional

from typeguard import typechecked

from ...helpers import Compatible, deprecate, format_result
from .queries import (GQL_CREATE_USER,
                      GQL_RESET_PASSWORD, GQL_UPDATE_API_KEY,
                      GQL_UPDATE_PASSWORD, GQL_UPDATE_PROPERTIES_IN_USER)


class MutationsUser:
    """
    Set of User mutations
    """
    # pylint: disable=too-many-arguments,too-many-locals

    def __init__(self, auth):
        """
        Initializes the subclass

        Parameters
        ----------
        - auth : KiliAuth object
        """
        self.auth = auth

    @Compatible(['v1', 'v2'])
    @typechecked
    def create_user(self, name: str, email: str, password: str, organization_role: str):
        """
        Add a user to your organization.

        Parameters
        ----------
        - name : str
            Name of the new user.
        - email : str
            Email of the new user, used as his unique identifier.
        - password : str
            On the first sign in, he will use this password and be able to change it.
        - organization_role : str
            One of "ADMIN", "USER".

        Returns
        -------
        - a result object which indicates if the mutation was successful, or an error message else.
        """
        variables = {
            'data': {'name': name,
                     'email': email,
                     'password': password,
                     'organizationRole': organization_role}
        }
        result = self.auth.client.execute(GQL_CREATE_USER, variables)
        return format_result('data', result)

    @Compatible(['v1', 'v2'])
    @typechecked
    def update_api_key(self, email: str, new_api_key: str):
        """
        Update API key

        Allows you to modify the API key you use to connect to Kili.
        This method is currently only active for Kili administrators.

        Parameters
        ----------
        - email : str
        - new_api_key : str

        Returns
        -------
        - a result object which indicates if the mutation was successful, or an error message else.
        """
        variables = {
            'data': {'newApiKey': new_api_key},
            'where': {'email': email}
        }
        result = self.auth.client.execute(GQL_UPDATE_API_KEY, variables)
        return format_result('data', result)

    @Compatible(['v1', 'v2'])
    @typechecked
    def update_password(self, email: str, old_password: str, new_password_1: str,
                        new_password_2: str):
        """
        Allows you to modify the password you use to connect to Kili.
        This resolver only works for on-premise installations without Auth0.

        Parameters
        ----------
        - email : str
        - old_password : str
        - new_password_1 : str
            The new password.
        - new_password_2 : str
            A confirmation field for the new password.

        Returns
        -------
        - a result object which indicates if the mutation was successful, or an error message else.
        """
        variables = {
            'data': {'oldPassword': old_password,
                     'newPassword1': new_password_1,
                     'newPassword2': new_password_2},
            'where': {'email': email}
        }
        result = self.auth.client.execute(GQL_UPDATE_PASSWORD, variables)
        return format_result('data', result)

    @Compatible(['v1', 'v2'])
    @typechecked
    def reset_password(self, email: str):
        """
        Reset password

        This resolver only works for on-premise installations without Auth0,
        if your organization allows Kili to send emails.

        Parameters
        ----------
        - email : str
            Email of the person whose password has to be reset.

        Returns
        -------
        - a result object which indicates if the mutation was successful, or an error message else.
        """
        variables = {'where': {'email': email}}
        result = self.auth.client.execute(GQL_RESET_PASSWORD, variables)
        return format_result('data', result)

    @Compatible(['v1', 'v2'])
    @typechecked
    def update_properties_in_user(self,
                                  email: str,
                                  name: Optional[str] = None,
                                  organization_id: Optional[str] = None,
                                  organization_role: Optional[str] = None,
                                  activated: Optional[bool] = None):
        """
        Update the properties of a user

        Parameters
        ----------
        - email : str
            The email is the identifier of the user
        - name : str, optional (default = None)
        - organization_id : str, optional (default = None)
            Change the organization the user is related to.
        - organization_role : str, optional (default = None)
            Change the role of the user. One of "ADMIN", "TEAM_MANAGER", "REVIEWER", "LABELER".
        - activated : bool, optional (default = None)
            In case we want to deactivate a user, but keep it.

        Returns
        -------
        - a result object which indicates if the mutation was successful, or an error message else.
        """
        variables = {
            'email': email,
        }
        if name is not None:
            variables['name'] = name
        if organization_id is not None:
            variables['organizationId'] = organization_id
        if organization_role is not None:
            variables['organizationRole'] = organization_role
        if activated is not None:
            variables['activated'] = activated
        result = self.auth.client.execute(
            GQL_UPDATE_PROPERTIES_IN_USER, variables)
        return format_result('data', result)



"""
Project mutations
"""

from json import dumps
from typing import Optional

from typeguard import typechecked

from ...helpers import Compatible, format_result, deprecate
from ...queries.asset import QueriesAsset
from ...queries.project import QueriesProject
from .queries import (GQL_APPEND_TO_ROLES,
                      GQL_CREATE_PROJECT,
                      GQL_DELETE_FROM_ROLES,
                      GQL_DELETE_PROJECT,
                      GQL_PROJECT_DELETE_ASYNCHRONOUSLY,
                      GQL_MAKE_PROJECT_PUBLIC,
                      GQL_GQL_UPDATE_PROPERTIES_IN_PROJECT_USER,
                      GQL_UPDATE_PROPERTIES_IN_PROJECT,
                      GQL_UPDATE_PROPERTIES_IN_ROLE)


class MutationsProject:

    """
    Set of Project mutations
    """
    # pylint: disable=too-many-arguments,too-many-locals

    def __init__(self, auth):
        """
        Initializes the subclass

        Parameters
        ----------
        - auth : KiliAuth object
        """
        self.auth = auth

    @Compatible(['v1', 'v2'])
    @typechecked
    def append_to_roles(self, project_id: str, user_email: str, role: str = 'LABELER'):
        """
        Add a user to a project

        If the user does not exist in your organization, he is invited and added
        both to your organization and project. This function can also be used to change
        the role of the user in the project.

        Parameters
        ----------
        - project_id : str
            Id of the project.
        - user_email : str
            The email of the user. This email is used as the unique identifier of the user.
        - role : str, optional (default = 'LABELER')
            One of {"ADMIN", "TEAM_MANAGER", "REVIEWER", "LABELER"}.

        Returns
        -------
        - a result object which indicates if the mutation was successful, or an error message else.

        Examples
        -------
        >>> kili.append_to_roles(project_id=project_id, user_email='john@doe.com')
        """
        variables = {
            'data': {'role': role,
                     'userEmail': user_email},
            'where': {'id': project_id}
        }
        result = self.auth.client.execute(GQL_APPEND_TO_ROLES, variables)
        return format_result('data', result)

    @Compatible(['v1', 'v2'])
    @typechecked
    def update_properties_in_project(self, project_id: str,
                                     consensus_mark: Optional[float] = None,
                                     consensus_tot_coverage: Optional[int] = None,
                                     description: Optional[str] = None,
                                     honeypot_mark: Optional[float] = None,
                                     instructions: Optional[str] = None,
                                     interface_category: str = 'IV2',
                                     input_type: Optional[str] = None,
                                     json_interface: Optional[dict] = None,
                                     min_consensus_size: Optional[int] = None,
                                     number_of_assets: Optional[int] = None,
                                     number_of_assets_with_empty_labels: Optional[int] = None,
                                     number_of_remaining_assets: Optional[int] = None,
                                     number_of_reviewed_assets: Optional[int] = None,
                                     review_coverage: Optional[int] = None,
                                     rules: Optional[list] = None,
                                     should_relaunch_kpi_computation: Optional[bool] = None,
                                     title: Optional[str] = None,
                                     use_honeypot: Optional[bool] = None):
        """
        Update properties of a project

        Parameters
        ----------
        - project_id : str
            Identifier of the project
        - consensus_mark : float, optional (default = None)
            Should be between 0 and 1
        - consensus_tot_coverage : int, optional (default = None)
            Should be between 0 and 100. It is the percentage of the dataset
            that will be annotated several times.
        - description : str, optional (default = None)
        - honeypot_mark : float, optional (default = None)
            Should be between 0 and 1
        - instructions : str, optional (default = None)
        - interface_category : str, optional (default = 'IV2')
            Always use 'IV2'
        - input_type : str, optional (default = None)
            Currently, one of {AUDIO, IMAGE, PDF, TEXT, URL, VIDEO, NA}
        - json_interface : dict, optional (default = None)
            The json parameters of the project, see Edit your interface.
        - min_consensus_size : int, optional (default = None)
            Number of people that will annotate the same asset, for consensus computation.
        - number_of_assets : int, optional (default = None)
            Defaults to 0
        - number_of_assets_with_empty_labels : int, optional (default = None)
            Defaults to 0
        - number_of_remaining_assets : int, optional (default = None)
            Defaults to 0
        - number_of_reviewed_assets : int, optional (default = None)
            Defaults to 0
        - review_coverage : int, optional (default = None)
            Allow to set the percentage of assets that will be queued in the review interface
        - rules : list, optional (default = None)
            Set basic rules to display a job in the label interface.
        - should_relaunch_kpi_computation : bool, optional (default = None)
            Technical field, added to indicate changes in honeypot or consensus settings
        - title : str, optional (default = None)
            Title of the project
        - use_honeypot : bool, optional (default = None)
            Activate / Desactivate the use of honeypot in the project

        Returns
        -------
        - a result object which indicates if the mutation was successful, or an error message else.

        Examples
        -------
        >>> kili.update_properties_in_project(project_id=project_id, title='New title')
        """
        variables = {
            'consensusMark': consensus_mark,
            'consensusTotCoverage': consensus_tot_coverage,
            'description': description,
            'honeypotMark': honeypot_mark,
            'instructions': instructions,
            'interfaceCategory': interface_category,
            'inputType': input_type,
            'jsonInterface': dumps(json_interface) if json_interface is not None else None,
            'minConsensusSize': min_consensus_size,
            'numberOfAssets': number_of_assets,
            'numberOfAssetsWithSkippedLabels': number_of_assets_with_empty_labels,
            'numberOfRemainingAssets': number_of_remaining_assets,
            'numberOfReviewedAssets': number_of_reviewed_assets,
            'projectID': project_id,
            'reviewCoverage': review_coverage,
            'rules': dumps(rules) if rules is not None else None,
            'shouldRelaunchKpiComputation': should_relaunch_kpi_computation,
            'title': title,
            'useHoneyPot': use_honeypot
        }
        result = self.auth.client.execute(
            GQL_UPDATE_PROPERTIES_IN_PROJECT, variables)
        return format_result('data', result)

    @Compatible(endpoints=['v2'])
    @typechecked
    def create_project(self, input_type: str, json_interface: dict,
                       title: str, description: str = '', project_type: Optional[str] = None,
                       user_id: Optional[str] = None):
        # pylint: disable=line-too-long
        """
        Create a project

        For more detailed examples on how to create projects,
        see [the recipe](https://github.com/kili-technology/kili-playground/blob/master/recipes/create_project.ipynb).

        Parameters
        ----------
        - input_type : str
            Currently, one of {AUDIO, IMAGE, PDF, TEXT, URL, VIDEO, NA}
        - json_interface: dict
            The json parameters of the project, see Edit your interface.
        - title : str
        - description : str, optional (default = '')
        - project_type: str, optional (default = None)
            Currently, one of {
                IMAGE_CLASSIFICATION_SINGLE,
                IMAGE_CLASSIFICATION_MULTI,
                IMAGE_OBJECT_DETECTION_RECTANGLE,
                IMAGE_OBJECT_DETECTION_POLYGON,
                IMAGE_OBJECT_DETECTION_SEMANTIC,
                OCR, PDF_CLASSIFICATION_SINGLE,
                PDF_CLASSIFICATION_MULTI,
                TEXT_CLASSIFICATION_SINGLE,
                TEXT_CLASSIFICATION_MULTI,
                TEXT_TRANSCRIPTION, TEXT_NER,
                VIDEO_CLASSIFICATION_SINGLE,
                VIDEO_FRAME_CLASSIFICATION,
                VIDEO_FRAME_OBJECT_TRACKING,
                SPEECH_TO_TEXT
            }
        - user_id : str, optional (default = None)

        Returns
        -------
        - a result object which indicates if the mutation was successful, or an error message else.

        Examples
        -------
        >>> kili.create_project(input_type='IMAGE', json_interface=json_interface, title='Example')
        """
        if user_id is None:
            user_id = self.auth.user_id
        variables = {
            'data': {'description': description,
                     'inputType': input_type,
                     'jsonInterface': dumps(json_interface),
                     'projectType': project_type,
                     'title': title}
        }
        result = self.auth.client.execute(GQL_CREATE_PROJECT, variables)
        return format_result('data', result)

    @Compatible(['v2'])
    @typechecked
    def make_project_public(self, project_id: str):
        """
        Make a project public. Warning: This action is permanent and irreversible.

        Parameters
        ----------
        - project_id : str

        Returns
        -------
        - the public token to provide in the public URL
        """
        variables = {
            'projectID': project_id
        }
        result = self.auth.client.execute(GQL_MAKE_PROJECT_PUBLIC, variables)
        project = format_result('data', result)
        return project['publicToken']

    @Compatible(['v1', 'v2'])
    @typechecked
    def update_properties_in_role(self, role_id: str, project_id: str, user_id: str, role: str):
        """
        Update properties of a role

        You should be either an admin or a team manager of the project, or an admin of the
        organization to be able to change the role of somebody.

        Parameters
        ----------
        - role_id : str
            Role identifier of the user. E.g. : 'to-be-deactivated'
        - project_id : str
            Identifier of the project
        - user_id : str
            The email or identifier of the user with updated role
        - role : str
            The new role. One of "ADMIN", "TEAM_MANAGER", "REVIEWER", "LABELER"

        Returns
        -------
        - a result object which indicates if the mutation was successful, or an error message else.
        """
        variables = {
            'roleID': role_id,
            'projectID': project_id,
            'userID': user_id,
            'role': role
        }
        result = self.auth.client.execute(
            GQL_UPDATE_PROPERTIES_IN_ROLE, variables)
        return format_result('data', result)

    @Compatible(['v1', 'v2'])
    @typechecked
    def delete_from_roles(self, role_id: str):
        """
        Delete users by their role_id

        Parameters
        ----------
        - role_id : str

        Returns
        -------
        - a result object which indicates if the mutation was successful, or an error message else.
        """
        variables = {'where': {'id': role_id}}
        result = self.auth.client.execute(GQL_DELETE_FROM_ROLES, variables)
        return format_result('data', result)

    @Compatible(['v2'])
    @typechecked
    def update_properties_in_project_user(self, project_user_id: str,
                                          consensus_mark: Optional[float] = None,
                                          honeypot_mark: Optional[float] = None,
                                          number_of_labeled_assets: Optional[int] = None,
                                          starred: Optional[bool] = None,
                                          total_duration: Optional[int] = None):
        """
        Update properties of a project-user tuple

        Parameters
        ----------
        - project_user_id : str
        - consensus_mark : float, optional (default = None)
            Should be between 0 and 1.
        - honeypot_mark : float, optional (default = None)
            Should be between 0 and 1.
        - number_of_labeled_assets : int, optional (default = None)
            Number of assets the user labeled in the project.
        - starred : bool, optional (default = None)
            Whether to star the project in the project list.
        - total_duration : int, optional (default = None)
            Total time the user spent in the project.

        Returns
        -------
        - a result object which indicates if the mutation was successful, or an error message else.

        Examples
        -------
        >>> for project_user in project_users:
        ...     kili.update_properties_in_project_user(
                    project_user_id=project_user['id'],
                    honeypot_mark=0)
        """
        variables = {
            'consensusMark': consensus_mark,
            'honeypotMark': honeypot_mark,
            'numberOfLabeledAssets': number_of_labeled_assets,
            'projectUserID': project_user_id,
            'starred': starred,
            'totalDuration': total_duration,
        }
        result = self.auth.client.execute(
            GQL_GQL_UPDATE_PROPERTIES_IN_PROJECT_USER, variables)
        return format_result('data', result)

    @Compatible()
    @typechecked
    def force_project_kpis(self, project_id: str):
        """
        Compute KPIs for a project

        Parameters
        ----------
        - project_id : str

        Returns
        -------
        - None
        """
        _ = QueriesAsset(self.auth).assets(project_id=project_id)
        _ = QueriesProject(self.auth).projects(project_id=project_id)

    @Compatible(['v1', 'v2'])
    @typechecked
    def internal_delete_project(self, project_id: str):
        """
        Delete project permanently.
        WARNING: This resolver is for internal use by Kili Technology only.

        Parameters
        ----------
        - project_id : str

        Returns
        -------
        - a result object which indicates if the mutation was successful, or an error message else.
        """
        variables = {'projectID': project_id}
        result = self.auth.client.execute(GQL_DELETE_PROJECT, variables)
        return format_result('data', result)

    @Compatible(['v1', 'v2'])
    @typechecked
    def delete_project(self, project_id: str):
        """
        Delete project permanently.

        Parameters
        ----------
        - project_id : str

        Returns
        -------
        - a result object which indicates if the mutation was successful, or an error message else.
        """
        variables = {'projectID': project_id}
        result = self.auth.client.execute(
            GQL_PROJECT_DELETE_ASYNCHRONOUSLY, variables)
        return format_result('data', result)



"""
Queries of asset mutations
"""

from .fragments import ASSET_FRAGMENT


GQL_APPEND_MANY_TO_DATASET = f'''
mutation(
    $data: AppendManyToDatasetData!,
    $where: ProjectWhere!
  ) {{
  data: appendManyToDataset(
    data: $data,
    where: $where
  ) {{
    {ASSET_FRAGMENT}
  }}
}}
'''


GQL_UPDATE_PROPERTIES_IN_ASSETS = f'''
mutation(
    $whereArray: [AssetWhere!]!
    $dataArray: [AssetData!]!
) {{
  data: updatePropertiesInAssets(
    where: $whereArray,
    data: $dataArray
  ) {{
    {ASSET_FRAGMENT}
  }}
}}
'''

GQL_DELETE_MANY_FROM_DATASET = f'''
mutation($where: AssetWhere!) {{
  data: deleteManyFromDataset(where: $where) {{
    {ASSET_FRAGMENT}
  }}
}}
'''



"""
Dataset mutations
"""

from typing import List, Optional
from functools import partial

from typeguard import typechecked

from ...helpers import (Compatible,
                        format_result)
from .queries import GQL_APPEND_DATASETS_TO_PROJECT, GQL_APPEND_TO_DATASET, \
    GQL_CREATE_DATASET, GQL_DELETE_DATASET, GQL_UPDATE_PROPERTIES_IN_DATASET


class MutationsDataset:
    """
    Set of Dataset mutations
    """

    def __init__(self, auth):
        """
        Initializes the subclass

        Parameters
        ----------
        - auth : KiliAuth object
        """
        self.auth = auth

    @Compatible(['v2'])
    @typechecked
    def append_to_dataset(self, content_array: List[str],
                          dataset_id: str,
                          external_id_array: List[str]):
        """
        Appends assets to a dataset.

        Parameters
        ----------
        - content_array : List[str]
            List of asset content to add
        - dataset_id : str
            Id of the target dataset
        - external_id_array : List[str]
            List of asset external ids to add

        Returns
        -------
        - a result object which indicates if the mutation was successful, or an error message else.

        Examples
        -------
        kili.append_to_dataset(
            content_array=['Hello world!'],
            dataset_id='ckg22d81r0jrg0885unmuswj8',
            external_ids=['text-1'])
        """

        variables = {
            'data': {'contentArray': content_array,
                     'externalIdArray': external_id_array},
            'where': {'id': dataset_id}
        }
        result = self.auth.client.execute(GQL_APPEND_TO_DATASET, variables)
        return format_result('data', result)

    @Compatible(['v2'])
    @typechecked
    def append_datasets_to_project(self, project_id: str,
                                   dataset_ids: List[str],
                                   dataset_asset_ids: List[str]):
        """
        Appends dataset assets to a project.

        Parameters
        ----------
        - project_id : str
            Id of the target project
        - dataset_ids : List[str]
            Ids of the source datasets
        - dataset_asset_ids : List[dict]
            List of asset ids

        Returns
        -------
        - a number indicating how many assets were added.

        Examples
        -------
        kili.append_datasets_to_project(
            project_id='ckg22d81s0jrh0885pdxfd03n',
            dataset_ids=['ckg22d81r0jrg0885unmuswj8'],
            dataset_asset_ids=['s0jrh0885unm1s0j0jrh0885p'])
        """

        variables = {
            'data': {'datasetIds': dataset_ids,
                     'datasetAssetIds': dataset_asset_ids},
            'where': {'id': project_id}
        }
        result = self.auth.client.execute(
            GQL_APPEND_DATASETS_TO_PROJECT, variables)
        return format_result('data', result)

    @Compatible(['v2'])
    @typechecked
    def create_dataset(self, name: str, asset_type: str):
        """
        Creates a dataset.

        Parameters
        ----------
        - name : str
            Id of the target project
        - asset_type : str
            Currently, one of {AUDIO, IMAGE, PDF, TEXT, URL, VIDEO, NA}

        Returns
        -------
        - a result object which indicates if the mutation was successful, or an error message else.

        Examples
        -------
        kili.create_dataset(
            name='My image dataset',
            asset_type='IMAGE')
        """

        variables = {
            'data': {'name': name,
                     'assetType': asset_type},
        }
        result = self.auth.client.execute(GQL_CREATE_DATASET, variables)
        return format_result('data', result)

    @Compatible(['v2'])
    @typechecked
    def delete_dataset(self, dataset_id: str):
        """
        Deletes a dataset.

        Parameters
        ----------
        - dataset_id : str
            Id of the dataset to delete

        Returns
        -------
        - a result object which indicates if the mutation was successful, or an error message else.

        Examples
        -------
        kili.delete_dataset(
            dataset_id='ckg22d81r0jrg0885unmuswj8')
        """

        variables = {
            'where': {'id': dataset_id},
        }
        result = self.auth.client.execute(GQL_DELETE_DATASET, variables)
        return format_result('data', result)

    @Compatible(['v2'])
    @typechecked
    def update_properties_in_dataset(self, dataset_id: str, asset_type: str, name: str):
        """
        Update properties in a dataset.

        Parameters
        ----------
        - dataset_id : str
            Id of the dataset to delete
        - asset_type : str
            Currently, one of {AUDIO, IMAGE, PDF, TEXT, URL, VIDEO, NA}
        - name : str
            Id of the target project

        Returns
        -------
        - a result object which indicates if the mutation was successful, or an error message else.

        Examples
        -------
        kili.update_properties_in_dataset(
            dataset_id='ckg22d81r0jrg0885unmuswj8',
            asset_type='IMAGE',
            name='Image dataset')
        """

        variables = {
            'assetType': asset_type,
            'datasetId': dataset_id,
            'name': name,
        }
        result = self.auth.client.execute(
            GQL_UPDATE_PROPERTIES_IN_DATASET, variables)
        return format_result('data', result)



"""
Project version queries
"""

from typing import List, Optional

from typeguard import typechecked

from ...helpers import Compatible, format_result, fragment_builder
from .queries import gql_project_version, GQL_PROJECT_VERSION_COUNT
from ...types import ProjectVersion as ProjectVersionType

class QueriesProjectVersion:
    """
    Set of ProjectVersion queries
    """
    # pylint: disable=too-many-arguments,too-many-locals

    def __init__(self, auth):
        """
        Initializes the subclass

        Parameters
        ----------
        - auth : KiliAuth object
        """
        self.auth = auth

    # pylint: disable=dangerous-default-value
    @Compatible(['v2'])
    @typechecked
    def project_version(
            self,
            first: Optional[int] = 100,
            skip: Optional[int] = 0,
            fields: List[str] = [
                'createdAt',
                'id',
                'content',
                'name',
                'project',
                'projectId'],
            project_id: str = None):
        # pylint: disable=line-too-long
        """
        Get an array of project version given a set of constraints

        Parameters
        ----------
        - fields : list of string, optional (default = ['createdAt', 'id', 'content',
            'name', 'project'])
            All the fields to request among the possible fields for the project versions
            See [the documentation](https://cloud.kili-technology.com/docs/python-graphql-api/graphql-api/#projectVersions) for all possible fields.
        - first : int (default = 100)
            Optional, Number of project versions to query
        - project_id : string (default = '')
            Filter on Id of project
        - skip : int (default = 0)
            Optional, number of project versions to skip (they are ordered by their date
            of creation, first to last).

        Returns
        -------
        - a result object which contains the query if it was successful, or an error message else.
        """
        formatted_first = first if first else 100
        variables = {
            'where': {
                'projectId': project_id,
            },
            'skip': skip,
            'first': formatted_first,
        }
        _gql_project_version = gql_project_version(
            fragment_builder(fields, ProjectVersionType))
        result = self.auth.client.execute(_gql_project_version, variables)
        return format_result('data', result)

    @Compatible(['v2'])
    @typechecked
    def count_project_versions(self, project_id: str):
        """
        Count the number of project versions

        Parameters
        ----------
        - project_id :
            Filter on ID of project

        Returns
        -------
        - the number of project versions with the parameters provided
        """
        variables = {
            'where': {'projectId': project_id},
        }
        result = self.auth.client.execute(GQL_PROJECT_VERSION_COUNT, variables)
        count = format_result('data', result)
        return count



"""
Queries of user mutations
"""

from .fragments import USER_FRAGMENT

GQL_CREATE_USER = f'''
mutation(
    $data: CreateUserData!
) {{
  data: createUser(
      data: $data
  ) {{
    {USER_FRAGMENT}
  }}
}}
'''

GQL_UPDATE_API_KEY = f'''
mutation(
    $data: UpdateApiKeyData!
    $where: UserWhere!
) {{
  data: updateApiKey(
    data: $data
    where: $where
  ) {{
    {USER_FRAGMENT}
  }}
}}
'''

GQL_UPDATE_PASSWORD = f'''
mutation(
    $data: UpdatePasswordData!
    $where: UserWhere!
) {{
  data: updatePassword(
    data: $data
    where: $where
  ) {{
    {USER_FRAGMENT}
  }}
}}
'''

GQL_RESET_PASSWORD = f'''
mutation($where: UserWhere!) {{
  data: resetPassword(where: $where) {{
    {USER_FRAGMENT}
  }}
}}
'''

GQL_UPDATE_PROPERTIES_IN_USER = f'''
mutation(
    $email: String!
    $name: String
    $organizationId: String
    $organizationRole: OrganizationRole
    $activated: Boolean
) {{
  data: updatePropertiesInUser(
    where: {{email: $email}}
    data: {{
      name: $name
      email: $email
      organizationId: $organizationId
      organizationRole: $organizationRole
      activated: $activated
    }}
  ) {{
    {USER_FRAGMENT}
  }}
}}
'''



"""
Dataset asset queries
"""


from typing import List, Optional

from typeguard import typechecked
import pandas as pd
from tqdm import tqdm

from ...helpers import Compatible, format_result, fragment_builder
from .queries import gql_assets, GQL_DATASET_ASSETS_COUNT
from ...types import DatasetAsset as DatasetAssetType
from ...orm import Asset


class QueriesDatasetAsset:
    """
    Set of DatasetAsset queries
    """
    # pylint: disable=too-many-arguments,too-many-locals

    def __init__(self, auth):
        """
        Initializes the subclass

        Parameters
        ----------
        - auth : KiliAuth object
        """
        self.auth = auth

    # pylint: disable=dangerous-default-value
    @Compatible(['v2'])
    @typechecked
    def dataset_assets(self, asset_id: Optional[str] = None, dataset_id: Optional[str] = None,
                       skip: int = 0,
                       fields: list = ['content', 'createdAt', 'externalId', 'id', 'jsonMetadata'],
                       disable_tqdm: bool = False,
                       first: Optional[int] = None):
        # pylint: disable=line-too-long
        """
        Get an array of dataset assets respecting a set of constraints

        Parameters
        ----------
        - asset_id : str, optional (default = None)
            The unique id of the asset to retrieve.
        - dataset_id : str
            Identifier of the dataset.
        - skip : int, optional (default = None)
            Number of assets to skip (they are ordered by their date of creation, first to last).
        - fields : list of string, optional (default = ['content', 'createdAt',
            'externalId', 'id', 'jsonMetadata'])
            All the fields to request among the possible fields for the assets.
            See [the documentation](https://cloud.kili-technology.com/docs/python-graphql-api/graphql-api/#datasetasset) for all possible fields.
        - first : int, optional (default = None)
            Maximum number of assets to return. Can only be between 0 and 100.
        - disable_tqdm : bool, optional (default = False)

        Returns
        -------
        - a result object which contains the query if it was successful, or an error message else.

        Examples
        -------
        >>> kili.dataset_assets(dataset_id=dataset_id)
        >>> kili.dataset_assets(asset_id=asset_id)
        """
        saved_args = locals()
        count_args = {k: v for (k, v) in saved_args.items()
                      if k not in ['skip', 'first', 'fields', 'self', 'disable_tqdm']}
        number_of_assets_with_search = self.count_dataset_assets(**count_args)
        total = min(number_of_assets_with_search,
                    first) if first is not None else number_of_assets_with_search
        formatted_first = first if first else 100
        if total == 0:
            return []
        with tqdm(total=total, disable=disable_tqdm) as pbar:
            paged_assets = []
            while True:
                variables = {
                    'where': {
                        'id': asset_id,
                        'dataset': {
                            'id': dataset_id,
                        },
                    },
                    'skip': skip,
                    'first': formatted_first,
                }
                _gql_assets = gql_assets(
                    fragment_builder(fields, DatasetAssetType))
                result = self.auth.client.execute(_gql_assets, variables)
                assets = format_result('data', result, Asset)
                if assets is None or len(assets) == 0 \
                        or (first is not None and len(paged_assets) == first):
                    return paged_assets
                if first is not None:
                    assets = assets[:max(0, first - len(paged_assets))]
                paged_assets += assets
                skip += formatted_first
                pbar.update(len(assets))

    @Compatible(['v2'])
    @typechecked
    def count_dataset_assets(
            self,
            asset_id: Optional[str] = None,
            dataset_id: Optional[str] = None):
        """
        Count and return the number of assets with the given constraints

        Parameters
        ----------
        - asset_id : str, optional (default = None)
            The unique id of the asset to retrieve.
        - dataset_id : str
            Identifier of the dataset.

        Returns
        -------
        - a result object which contains the query if it was successful, or an error message else.

        Examples
        -------
        >>> kili.count_dataset_assets(dataset_id=dataset_id)
        23
        >>> kili.count_dataset_assets(asset_id=asset_id)
        1
        """
        variables = {
            'where': {
                'id': asset_id,
                'dataset': {
                    'id': dataset_id,
                },
            }
        }
        result = self.auth.client.execute(GQL_DATASET_ASSETS_COUNT, variables)
        count = format_result('data', result)
        return count



"""
Queries of organization mutations
"""

from .fragments import ORGANIZATION_FRAGMENT

GQL_CREATE_ORGANIZATION = f'''
mutation(
    $data: CreateOrganizationData!
) {{
  data: createOrganization(
    data: $data
  ) {{
    {ORGANIZATION_FRAGMENT}
  }}
}}
'''

GQL_UPDATE_PROPERTIES_IN_ORGANIZATION = f'''
mutation(
    $id: ID!
    $name: String
    $address: String
    $license: String
    $zipCode: String
    $city: String
    $country: String
) {{
  data: updatePropertiesInOrganization(
    where: {{id: $id}}
    data: {{
      name: $name
      address: $address
      license: $license
      zipCode: $zipCode
      city: $city
      country: $country
    }}
  ) {{
    {ORGANIZATION_FRAGMENT}
  }}
}}
'''
